Package: MultimodalExperiment
Title: Integrative Bulk and Single-Cell Experiment Container
Version: 1.13.0
Authors@R: 
    person(given = "Lucas",
           family = "Schiffer",
           role = c("aut", "cre"),
           email = "schiffer.lucas@gmail.com",
           comment = c(ORCID = "0000-0003-3628-0326"))
Description: MultimodalExperiment is an S4 class that integrates bulk
        and single-cell experiment data; it is optimally
        storage-efficient, and its methods are exceptionally fast. It
        effortlessly represents multimodal data of any nature and
        features normalized experiment, subject, sample, and cell
        annotations, which are related to underlying biological
        experiments through maps. Its coordination methods are opt-in
        and employ database-like join operations internally to deliver
        fast and flexible management of multimodal data.
License: Artistic-2.0
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.3.0), IRanges, S4Vectors
Imports: BiocGenerics, MultiAssayExperiment, methods, utils
Suggests: BiocStyle, knitr, rmarkdown
biocViews: DataRepresentation, Infrastructure, SingleCell
VignetteBuilder: knitr
RoxygenNote: 7.3.1
Roxygen: list(markdown = TRUE)
Config/pak/sysreqs: libicu-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 13:00:52 UTC
RemoteUrl: https://github.com/bioc/MultimodalExperiment
RemoteRef: HEAD
RemoteSha: c54ec6c109df25cb2afca369a0daf56e4f49f6b5
NeedsCompilation: no
Packaged: 2026-05-30 09:40:12 UTC; root
Author: Lucas Schiffer [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3628-0326>)
Maintainer: Lucas Schiffer <schiffer.lucas@gmail.com>
Built: R 4.6.0; ; 2026-05-30 13:52:16 UTC; unix
