# Generated by roxygen2: do not edit by hand

export("bulkExperiments<-")
export("cellData<-")
export("cellMap<-")
export("experiment<-")
export("experimentData<-")
export("experimentMap<-")
export("experimentNames<-")
export("experiments<-")
export("sampleData<-")
export("sampleMap<-")
export("singleCellExperiments<-")
export("subjectData<-")
export("subjectMap<-")
export(ExperimentList)
export(MultimodalExperiment)
export(bulkExperiments)
export(cellData)
export(cellMap)
export(experiment)
export(experimentData)
export(experimentMap)
export(experimentNames)
export(experiments)
export(joinAnnotations)
export(joinMaps)
export(sampleData)
export(sampleMap)
export(singleCellExperiments)
export(subjectData)
export(subjectMap)
exportClasses(MultimodalExperiment)
exportMethods("[")
exportMethods("[<-")
exportMethods("bulkExperiments<-")
exportMethods("cellData<-")
exportMethods("cellMap<-")
exportMethods("colnames<-")
exportMethods("dimnames<-")
exportMethods("experiment<-")
exportMethods("experimentData<-")
exportMethods("experimentMap<-")
exportMethods("experimentNames<-")
exportMethods("experiments<-")
exportMethods("names<-")
exportMethods("rownames<-")
exportMethods("sampleData<-")
exportMethods("sampleMap<-")
exportMethods("singleCellExperiments<-")
exportMethods("subjectData<-")
exportMethods("subjectMap<-")
exportMethods(bulkExperiments)
exportMethods(cellData)
exportMethods(cellMap)
exportMethods(colnames)
exportMethods(dimnames)
exportMethods(experiment)
exportMethods(experimentData)
exportMethods(experimentMap)
exportMethods(experimentNames)
exportMethods(experiments)
exportMethods(harmonize)
exportMethods(joinAnnotations)
exportMethods(joinMaps)
exportMethods(names)
exportMethods(propagate)
exportMethods(rownames)
exportMethods(sampleData)
exportMethods(sampleMap)
exportMethods(show)
exportMethods(singleCellExperiments)
exportMethods(subjectData)
exportMethods(subjectMap)
importClassesFrom(MultiAssayExperiment,ExperimentList)
importClassesFrom(S4Vectors,Annotated)
importClassesFrom(S4Vectors,DataFrame)
importFrom(BiocGenerics,"%in%")
importFrom(BiocGenerics,"colnames<-")
importFrom(BiocGenerics,"rownames<-")
importFrom(BiocGenerics,Map)
importFrom(BiocGenerics,Reduce)
importFrom(BiocGenerics,as.data.frame)
importFrom(BiocGenerics,cbind)
importFrom(BiocGenerics,colnames)
importFrom(BiocGenerics,duplicated)
importFrom(BiocGenerics,intersect)
importFrom(BiocGenerics,lapply)
importFrom(BiocGenerics,match)
importFrom(BiocGenerics,ncol)
importFrom(BiocGenerics,nrow)
importFrom(BiocGenerics,order)
importFrom(BiocGenerics,paste)
importFrom(BiocGenerics,rbind)
importFrom(BiocGenerics,rownames)
importFrom(BiocGenerics,setdiff)
importFrom(BiocGenerics,union)
importFrom(BiocGenerics,unique)
importFrom(BiocGenerics,unlist)
importFrom(IRanges,CharacterList)
importFrom(MultiAssayExperiment,ExperimentList)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,complete.cases)
importFrom(S4Vectors,mendoapply)
importFrom(S4Vectors,merge)
importFrom(S4Vectors,split)
importFrom(S4Vectors,unname)
importFrom(methods,"slot<-")
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(utils,head)
importFrom(utils,tail)
